@@ -1,3 +1,4 @@
+from datetime import datetime
 from enum import Enum
 from typing import Dict, List, Optional, Union
 from pydantic import root_validator, validator, constr
@@ -41,6 +42,24 @@ class GateType(str, Enum):
     Behavior of those gates is not implemented by the controller
     """
 
+    coupled_lane = "coupled_lane"
+    """
+    Gates which open always together with another lane (e.g. garage door after entry barrier) but can also be controlled
+    independently, e.g. perma open
+    """
+
+    multilane = "multilane"
+    """
+    Gates where multiple vehicles can pass at the same (either in parallel or in opposite direction).
+    This is only allowed in Freeflow mode and with no user interaction (e.g. display)
+    """
+
+    transit_multilane = "transit_multilane"
+    """
+    Multilane type of transit_custom (multiple vehicles can pass at the same). Only allowed for automatically setting
+    something for passing vehicles (e.g. changing rate) 
+    """
+
 
 class RefsModel(SettingsModel):
     """
@@ -59,6 +78,7 @@ class Gate(SettingsModel):
         Field(..., description="Technical name of gate")
     name: str = Field(..., description="Name for UIs", max_length=126)
     type: GateType = Field(..., description="Type of the gate. Defines the usage and general behavior")
+    opposite_gate: Optional[str] = Field(None, description="Name of the opposite gate for bidirectional gates")
 
 
 class Rate(SettingsModel):
@@ -113,14 +133,20 @@ class CommandCostEntryTypes(str, Enum):
     payoff_costs = "payoff_costs"
     """generates pay off over (set to 0 remaining when session not editable) for each group"""
 
+    free_timerange = "free_timerange"
+    """
+    instead of calculating dynamic costs from start to end, 
+    we calculate them from start to free_timerange start + free_timerange end to end
+    """
+
 
 class CostEntryData(SettingsModel):
     """
     Defines a cost entry command, which is for example set at entering the parking lot
     """
     entry_type: CommandCostEntryTypes = Field(..., description="Cost entry command type")
-    value: Optional[Union[int, str]] = Field(None,
-                                             description="value: Dependent on entry_type it's an int, a str or None")
+    value: Optional[Union[int, str, Dict[str, Optional[datetime]]]] = (
+        Field(None, description="value: Dependent on entry_type it's an int, a str or None"))
     group: Optional[str] = Field(None, description="Cost group the command cost entry is applied on. None for global")
     account_id: str = Field(..., description="The account_id the generated booked cost entry will be booked at. "
                                              "It is an integer between 0 and 3999 formatted as string")
@@ -128,9 +154,9 @@ class CostEntryData(SettingsModel):
     account_id_validator = account_id_range_validator("account_id", 0, 3999)
 
     # Not used in settings, set by service applying the command cost entry (e.g. access service)
-    # source: Optional[str]
-    # source_id: Optional[str]
-    # idempotency_key: Optional[str]
+    source: Optional[str]
+    source_id: Optional[str]
+    idempotency_key: Optional[str]
 
     @root_validator(skip_on_failure=True)
     def data_validation(cls, values):
@@ -143,13 +169,18 @@ class CostEntryData(SettingsModel):
                                           CommandCostEntryTypes.partial_rate_change]:
             values["value"] = str(values["value"])
         if values.get("entry_type") not in [CommandCostEntryTypes.payment, CommandCostEntryTypes.cancel_remaining_costs,
-                                            CommandCostEntryTypes.payoff_costs]:
+                                            CommandCostEntryTypes.payoff_costs, CommandCostEntryTypes.free_timerange]:
             if values["group"] is None:
                 raise ValueError("group must be set")
         elif values.get("entry_type") in [CommandCostEntryTypes.cancel_remaining_costs,
                                           CommandCostEntryTypes.payoff_costs]:
             if values["group"] is not None:
                 raise ValueError("group must NOT be set")
+        elif values.get("entry_type") == CommandCostEntryTypes.free_timerange:
+            values["value"] = dict(entry_start=values["value"].get("entry_start"),
+                                   entry_end=values["value"].get("entry_end"))
+            if values["group"] is None:
+                raise ValueError("group must be set")
         return values
 
 
@@ -189,6 +220,8 @@ class ParkingArea(SettingsModel):
     shortterm_limit_type: ShorttermLimitType = Field(..., description="Type of shortterm limit")
     shortterm_limit: int = Field(..., description="Limit value in this area. Compared to value specified in "
                                                   "shortterm_limit_type", ge=0)
+    number_of_parking_spaces: Optional[int] = Field(None, description="Number of parking spaces in this area. None if "
+                                                                      "not specified", ge=0)
 
     id_validator = uuid_validator("id")
 
@@ -383,12 +416,14 @@ class GarageSettings(SettingsModel):
     """
     mode: GarageModes = Field(..., description="General parking lot mode, e.g. freeflow. This field is more for "
                                                "information in UI and setup than for real technical use.")
-    honest_payment_enabled: bool = Field(..., description="If honest payments are allowed")
-    enforcement_enabled: bool = Field(..., description="If unpaid parking sessions can be enforced")
-    payment_possible: bool = Field(..., description="If any parking system payments (Kiosk, PayApp,...) are possible")
+    honest_payment_enabled: bool = Field(False, description="If honest payments are allowed")
+    enforcement_enabled: bool = Field(False, description="If unpaid parking sessions can be enforced")
+    payment_possible: bool = Field(False, description="If any parking system payments (Kiosk, PayApp,...) are possible")
 
     @root_validator(skip_on_failure=True, pre=True)
     def data_validation(cls, values):
+        if not isinstance(values, dict):
+            return values
         if values["mode"] == GarageModes.freeflow:
             values["honest_payment_enabled"] = True
             values["enforcement_enabled"] = True
@@ -415,7 +450,8 @@ class GarageName(SettingsModel):
     name: str = Field(..., description="Name of Garage for UIs and end users", max_length=126)
     technical_name: constr(min_length=3, max_length=63, regex=r"^[a-z0-9\-]+$") = \
         Field(..., description="Technical/internal name of garage")
-    slug: str = Field(..., description="Shortname of Garage used in URLs")
+    slug: constr(min_length=3, max_length=126, regex=r"^[a-zA-Z0-9\-_]+$") = \
+        Field(..., description="Shortname of Garage used in URLs")
 
 
 class Urls(SettingsModel):
