@@ -1,10 +1,11 @@
 from enum import Enum
+from typing import Optional
 
-from settings_models.validators import hhmm_validator
 from settings_models._combat import SettingsModel, Field
+from settings_models.validators import hhmm_validator
 
 
-class GateMode(str, Enum):
+class _GateMode(str, Enum):
     """
     How the gate should behave
     """
@@ -14,7 +15,7 @@ class GateMode(str, Enum):
     standard = "standard"
 
 
-class DayMode(SettingsModel):
+class _DayMode(SettingsModel):
     """
     If day mode (everyone can enter) is enabled and at what time of day
     """
@@ -24,3 +25,9 @@ class DayMode(SettingsModel):
 
     start_validator = hhmm_validator("start")
     end_validator = hhmm_validator("end")
+
+
+GateMode = Optional[_GateMode]
+GateMode.__doc__ = _GateMode.__doc__
+DayMode = Optional[_DayMode]
+DayMode.__doc__ = _DayMode.__doc__
